@@ -2,15 +2,14 @@ from copy import copy
 from typing import Tuple, Union
 
 import numpy as np
-from pyopencl.array import Array, empty, zeros, to_device
 from pytest import mark
 
-from pyopencl_extension import Thread, Helpers, Kernel, Program, Global, Scalar, Types
+from pyopencl_extension import Helpers, Kernel, Program, Global, Scalar, Types, Array, empty, zeros, to_device
 
-__author__ = "piveloper"
-__copyright__ = "26.03.2020, piveloper"
+__author__ = "P.Mohr"
+__copyright__ = "26.03.2020, P.Mohr"
 __version__ = "1.0"
-__email__ = "piveloper@gmail.com"
+__email__ = "philipp.mohr@tuhh.de"
 __doc__ = """This script implements the functionality to copy certain region_in of cl array on device."""
 
 from pyopencl_extension.types.utilities_np_cl import c_name_from_dtype
@@ -51,6 +50,10 @@ def test_slice():
                    (2, 0, None, 1))
     res = Slice[0:0]
     assert res == ((0, 0, 0, 1),)
+    res = Slice[:0]
+    assert res == ((0, 0, 0, 1),)
+    res = Slice[:, :0]
+    assert res == ((0, 0, 0, 1),)
 
 
 class CopyArrayRegion:
@@ -107,7 +110,6 @@ class CopyArrayRegion:
         _region_in_original = copy(region_in)  # for debug purposes
         _region_out_original = copy(region_out)  # for debug purposes
 
-        queue = in_buffer.queue
         if region_in is not None:
             region_in = self._deal_with_incomplete_regions(region_in, in_buffer)
 
@@ -132,7 +134,7 @@ class CopyArrayRegion:
 
         if out_buffer is None and region_out is None:
             shape = [ax[1] - ax[0] for ax in self.region_in]
-            self.out_buffer = empty(queue, tuple(shape), dtype=self.in_buffer.dtype)
+            self.out_buffer = empty(tuple(shape), dtype=self.in_buffer.dtype)
             self.region_out = [(0, i, 1) for i in shape]  # (tuple([0]*len(shape)),shape)
         elif out_buffer is not None and region_out is not None:
             self.out_buffer = out_buffer
@@ -148,16 +150,15 @@ class CopyArrayRegion:
 
         self.copy_array_region = Kernel(name='copy_array_region',
                                         args={'in_buffer': Global(self.in_buffer, read_only=True),
-                                                'out_buffer': Global(self.out_buffer, )},
+                                              'out_buffer': Global(self.out_buffer, )},
                                         body=["""
                                   int addr_in = ${command_addr_in};
                                   int addr_out = ${command_addr_out};
                                   out_buffer[addr_out]=in_buffer[addr_in];
                                   """],
                                         replacements={'command_addr_in': self._command_for_addr_in_computation(),
-                                                        'command_addr_out': self._command_for_addr_out_computation()},
-                                        global_size=self.shape_region_out).compile(
-            thread=Thread.from_buffer(in_buffer))
+                                                      'command_addr_out': self._command_for_addr_out_computation()},
+                                        global_size=self.shape_region_out).compile()
 
     def __call__(self, *args, **kwargs):
         self.copy_array_region()
@@ -199,13 +200,13 @@ def cl_set(array: Array, region: TypeSliceFormatCopyArrayRegion, value):
     if any([(part[0] - part[1]) == 0 for part in region]):  # check that there is no empty slice
         return
     global_size = np.product([part[1] - part[0] for part in region])
-    target_shape = to_device(array.queue, np.array(array.shape).astype(Types.int))
-    offset_target = to_device(array.queue, np.array([part[0] for part in region]).astype(Types.int))
-    source_shape = to_device(array.queue, np.array([part[1] - part[0] for part in region]).astype(Types.int))
+    target_shape = to_device(np.array(array.shape).astype(Types.int))
+    offset_target = to_device(np.array([part[0] for part in region]).astype(Types.int))
+    source_shape = to_device(np.array([part[1] - part[0] for part in region]).astype(Types.int))
     source_n_dims = len(source_shape)
 
     if isinstance(value, np.ndarray):
-        source = to_device(array.queue, value.astype(array.dtype))
+        source = to_device(value.astype(array.dtype))
         arg_source = Global(source)
         code_source = 'source[get_global_id(0)]'
     else:
@@ -215,12 +216,12 @@ def cl_set(array: Array, region: TypeSliceFormatCopyArrayRegion, value):
 
     knl = Kernel('set_cl_array',
                  {'target': Global(array),
-                    'target_shape': Global(target_shape),
-                    'offset_target': Global(offset_target),
-                    'source': arg_source,
-                    'source_shape': Global(source_shape),
-                    'source_n_dims': Scalar(Types.int)},
-                   """
+                  'target_shape': Global(target_shape),
+                  'offset_target': Global(offset_target),
+                  'source': arg_source,
+                  'source_shape': Global(source_shape),
+                  'source_n_dims': Scalar(Types.int)},
+                 """
    // id_source = get_global_id(0)
    // id_source points to element of array source which replaces element with id_target in array target.
    // we need to compute id_target from id_source:
@@ -234,11 +235,11 @@ def cl_set(array: Array, region: TypeSliceFormatCopyArrayRegion, value):
    // Finally, we can determine the id of the target array and copy element to corresponding position:
    // id_target = (id0*offset0t)*s1t*s2t ... (sxt: shape of target array along dim x)
    int id_target = 0; // to be iteratively computed from global id, slice dimensions and ary dimensions
-   
+
    int temp = get_global_id(0);
    int prod_source_id_multiplier = 1;
    int prod_target_id_multiplier = 1;
-   
+
    for(int i=source_n_dims-1; i>=0; i--){ // i=i_axis_source
     int id_source = temp % source_shape[i];
     temp = (int)((temp-id_source)/source_shape[i]);
@@ -249,9 +250,9 @@ def cl_set(array: Array, region: TypeSliceFormatCopyArrayRegion, value):
    target[id_target] = ${source};
                    """,
                  replacements={'addr': Helpers.command_compute_address(array.ndim),
-                                 'source': code_source},
+                               'source': code_source},
                  global_size=(global_size,)
-                 ).compile(Thread.from_buffer(array), emulate=False)
+                 ).compile(array.context, emulate=False)
     knl(source=source, source_n_dims=source_n_dims)
 
 
@@ -261,8 +262,7 @@ def cl_set(array: Array, region: TypeSliceFormatCopyArrayRegion, value):
                         [4, -1, 0, 3],
                         [0, 10, 0, 3]])
 def test_cl_set(s):
-    thread = Thread()
-    ary = zeros(thread.queue, (10, 3), Types.double)
+    ary = zeros((10, 3), Types.double)
     ary_np = ary.get()
     val = 2
     # ClSet(ary)[:, 2:3] = val
@@ -273,8 +273,7 @@ def test_cl_set(s):
 
 
 def test_cl_set_out_of_bounds():
-    thread = Thread()
-    ary = zeros(thread.queue, (10, 3), Types.double)
+    ary = zeros((10, 3), Types.double)
     val = 2
     try:
         cl_set(ary, Slice[:100, :], val)
@@ -284,8 +283,7 @@ def test_cl_set_out_of_bounds():
 
 def test_cl_set_many_dimensions():
     s = [0, 1, 2, 3, 0, 1]
-    thread = Thread()
-    ary = zeros(thread.queue, (10, 3, 4), Types.double)
+    ary = zeros((10, 3, 4), Types.double)
     ary_np = ary.get()
     val = 2
     # ClSet(ary)[:, 2:3] = val
@@ -297,21 +295,20 @@ def test_cl_set_many_dimensions():
 class TypeConverter:
     def __init__(self, in_buffer: Array, out_buffer_dtype: np.dtype):
         self.in_buffer = in_buffer
-        self.out_buffer = empty(in_buffer.queue, in_buffer.shape, dtype=out_buffer_dtype)
+        self.out_buffer = empty(in_buffer.shape, out_buffer_dtype, in_buffer.queue)
         knl = Kernel(name='type',
                      args={'in_buffer': Global(self.in_buffer, read_only=True),
-                             'out_buffer': Global(self.out_buffer)},
+                           'out_buffer': Global(self.out_buffer)},
                      body=["""
                                    int addr_in = ${command_addr_in};
                                    int addr_out = ${command_addr_out};
                                    out_buffer[addr_out]=convert_${buff_out_t}(in_buffer[addr_in]);
                                    """],
                      replacements={'command_addr_in': Helpers.command_compute_address(self.in_buffer.ndim),
-                                     'command_addr_out': Helpers.command_compute_address(self.out_buffer.ndim),
-                                     'buff_out_t': c_name_from_dtype(self.out_buffer.dtype)},
+                                   'command_addr_out': Helpers.command_compute_address(self.out_buffer.ndim),
+                                   'buff_out_t': c_name_from_dtype(self.out_buffer.dtype)},
                      global_size=self.in_buffer.shape)
-        thread = Thread(queue=in_buffer.queue, context=in_buffer.context)
-        self.program = Program(kernels=[knl]).compile(thread=thread, emulate=False)
+        self.program = Program(kernels=[knl]).compile(context=in_buffer.context, emulate=False)
 
     def __call__(self):
         self.program.type()
