@@ -1,5 +1,4 @@
 import inspect
-import logging
 import os
 import re
 from collections import namedtuple
@@ -11,24 +10,25 @@ from typing import Tuple, List
 import numpy as np
 from pycparser.c_ast import BinaryOp, ID, IdentifierType, Return, Constant, Assignment, Decl, FuncDef, Node, TypeDecl, \
     FuncCall, ExprList, Compound, ArrayRef, For, If, Cast, ArrayDecl, UnaryOp, While, Break, InitList, Typedef, \
-    StructRef, PtrDecl, TernaryOp
+    StructRef, PtrDecl, TernaryOp, EmptyStatement
 from pycparserext.ext_c_parser import FuncDeclExt, PreprocessorLine, OpenCLCParser
 
-__author__ = "piveloper"
-__copyright__ = "26.03.2020, piveloper"
+__author__ = "P.Mohr"
+__copyright__ = "26.03.2020, P.Mohr"
 __version__ = "1.0"
-__email__ = "piveloper@gmail.com"
+__email__ = "philipp.mohr@tuhh.de"
 __doc__ = """This script includes helpful functions to extended PyOpenCl functionality."""
 
 # make sure emulation does not import anything from framework, to avoid circular imports
 # framework does depends on emulation and not the other way around.
 from pyopencl._cl import LocalMemory
 
-from pyopencl.array import Array
-from pytest import mark
-
-from pyopencl_extension.types.funcs_for_emulation import CArray, CArrayVec, init_array
+from pyopencl_extension.types.funcs_for_emulation import init_array, CPointer, CPointerVec
 from pyopencl_extension.helpers.general import write_string_to_file
+from pyopencl_extension import Array
+from pyopencl.array import Array as ClArray
+
+arrays_cls = (Array, ClArray)
 
 # following lines are used to support functions from <pyopencl-complex.h>
 from pyopencl_extension.types.utilities_np_cl import is_vector_type
@@ -445,6 +445,8 @@ def _unparse(node: Node) -> Container:
         res = f'{_unparse(node.name)}.{_unparse(node.field)}'
     elif isinstance(node, TernaryOp):
         res = f'{_unparse(node.iftrue)} if {_unparse(node.cond)} else {_unparse(node.iffalse)}'
+    elif isinstance(node, EmptyStatement):
+        res = 'pass'
     elif node is None:
         res = ''
     else:
@@ -472,18 +474,18 @@ translation_cl_work_item_functions = {
 
 @dataclass
 class WorkItem:
-    global_id: Tuple[int]
-    global_size: Tuple[int]
-    local_size: Tuple[int]
-    local_memory_collection: List[dict]
+    global_id: tuple[int]
+    global_size: tuple[int]
+    local_size: tuple[int]
+    local_memory_collection: list[dict]
     _local: dict = field(init=False, default=None)
     _group_id_lin: int = None  #
     _scope: dict = None
-    work_items: List['WorkItem'] = None
-    local_id: Tuple[int] = None
-    num_groups: Tuple[int] = None
-    group_id: Tuple[int] = None
-    work_dim: Tuple[int] = None
+    work_items: list['WorkItem'] = None
+    local_id: tuple[int] = None
+    num_groups: tuple[int] = None
+    group_id: tuple[int] = None
+    work_dim: int = None
 
     def __post_init__(self):
         n_dim = self.get_work_dim()
@@ -577,7 +579,7 @@ def cl_kernel(kernel):
     """
 
     def wrapper_loop_over_grid(global_size, local_size=None, *args):
-        args_python = [arg.get().ravel() if isinstance(arg, Array) else arg for arg in args]
+        args_python = [arg.get().ravel() if isinstance(arg, arrays_cls) else arg for arg in args]
         num_work_groups = 1
         if local_size is None:
             local_size = global_size
@@ -606,9 +608,9 @@ def cl_kernel(kernel):
 
         def decide_ary_view(arg):
             if is_vector_type(arg.dtype):
-                return arg.view(CArrayVec)
+                return CPointerVec.from_np(arg)
             else:
-                return arg.view(CArray)
+                return CPointer.from_np(arg)
 
         args_python = [decide_ary_view(arg) if isinstance(arg, np.ndarray) else arg
                        for arg in args_python]
@@ -633,7 +635,8 @@ def cl_kernel(kernel):
                     [next(knl) for knl in blocking_kernels]
                 except StopIteration:
                     break
-        [arg.set(args_python[idx]) if isinstance(arg, Array) else arg for idx, arg in enumerate(args)]
+        [arg.set(args_python[idx].memory.reshape(arg.shape)) if isinstance(arg, arrays_cls) else arg
+         for idx, arg in enumerate(args)]
 
     return wrapper_loop_over_grid
 
@@ -645,7 +648,9 @@ regex_func_def = re.compile(r'#define[ ]+([\w]+)\(([\w, ]+)\)[ ]+([\w\.\(\)*\-+/
 # #define DFT2_TWIDDLE(a,b,t) { data_t tmp = t(a-b); a += b; b = tmp; }
 regex_func_def_multi_line = re.compile(r'(#define)([ ]+)([\w]+)(\()([\w, ]+)(\))([ ]+)(\{)(.+)(\})')
 regex_include = re.compile(r'#include[ ]+<([\w\-\.]+>)')  # '#include <pyopencl-complex.h>\n
-regex_numbers_with_conversion_characater = re.compile(r'([\d.]+)([fd])')
+regex_numbers_with_conversion_characater = re.compile(r'([\d.]+)([fd])')  # define PI 3.14159265359f
+# define SOME_ARRAY (int[]){0, 1, 2, 3}
+regex_array_definition = re.compile(r'#define[ ]+([\w]+)[ ]+\((.+)\[\]\)+\{(.+)\}')
 
 search = lambda regex, text: re.search(regex, text)
 
@@ -688,6 +693,9 @@ def unparse_preprocessor_line(line: PreprocessorLine) -> str:
     elif search(regex_include, contents):
         # until now no case occured where python needs to include something like opencl
         return ''
+    elif res := search(regex_array_definition, contents):
+        name, dtype, data = res.group(1), res.group(2), res.group(3)
+        return f'{name} = np.array([{data}], dtype={dtype})'
     else:
         raise ValueError('Preprocessing line content not covered with Regex')
 
@@ -742,17 +750,53 @@ def search_for_barrier(code_c, ast):
     funcs_with_barrier = [_ for _ in funcs if
                           any(_['start_line'] < line < _['end_line'] for line in lines_with_barrier)]
     funcs_nested_barrier = [_ for _ in funcs if any(re.search(r'(\W)(' + func['name'] + r')(\W)',
-                                                              ''.join(code_c[_['start_line']:_['end_line']-1]))
+                                                              ''.join(code_c[_['start_line']:_['end_line'] - 1]))
                                                     for func in funcs_with_barrier)]
 
     return [_['name'] for _ in funcs_with_barrier + funcs_nested_barrier if not _['is_kernel']]
 
 
+def replace_builtin_macros(code_c):
+    """
+    remove macros with builtin function calls and replace usages:
+
+    if macros are built in functions, this method performs the replacement where those macros are used.
+
+    the motivation for this solution is that built-in function are accessed via the work item object wi, which cannot
+    be called at the location of definement.
+
+    #define gid0 get_global_id(0)
+    #define gid1 get_global_id(1)
+    #define gid2 get_global_id(2)
+
+    // the regex search matches against the listed cases
+    __kernel void my_knl(__global int *a)
+    {
+        a[gid0]+=1.0;
+         get_gid0_
+         gid0
+         gid0*
+         gid0/
+         =gid0)
+         gid0==
+        ==gid0
+    }
+    """
+    rgx = '#define[ ]+([\w]+)[ ]+(get_[\w\.\-e\(\)]+)[\s\n]'
+    defines_to_be_replaced = re.findall(rgx, code_c)
+    code_c = re.sub(rgx, '', code_c)
+    for name, val in defines_to_be_replaced:
+        code_c = re.sub('([\[\(\s\+\*\/\-\=]|==)' + f'({name})' + '([\]\)\s\+\*\/\-\;]|==)', r'\1' + val + r'\3', code_c)
+    return code_c
+
+
 def unparse_c_code_to_python(code_c: str) -> str:
     # todo prevents files: https://stackoverflow.com/questions/12644902/how-to-prevent-table-regeneration-in-ply
     # yacc.yacc(debug=False, write_tables=False)
     code_c = re.sub('#define[ ]+TP_ROOT[ ]+(cfloat|cdouble])[ ]*(\n)', '', code_c)  # removes TP_ROOT = cfloat
 
+    code_c = replace_builtin_macros(code_c)
+
     p = OpenCLCParser(lex_optimize=False, yacc_optimize=False)
     os.remove('yacctab.py')
     # remove block comments like /* some comment */ since other p.parse throws parsing error
@@ -851,7 +895,7 @@ def create_py_file_and_load_module(code_py: str, file: str = None):
     return program_python
 
 
-def compute_linear_idx(idx_tuple, dimensions)->int:
+def compute_linear_idx(idx_tuple, dimensions) -> int:
     idx = 0
     n_dim = len(dimensions)
     for i in range(n_dim):
@@ -868,14 +912,4 @@ def compute_tuple_idx(idx_lin, dimensions):
     for _i in range(n_dim):
         i = n_dim - 1 - _i
         idx_lin, idx_tuple[i] = divmod(idx_lin, dimensions[i])
-    return tuple(idx_tuple)
-
-
-@mark.parametrize('idx_tuple,dimensions, idx_lin_ref', [((1, 2, 0), (2, 4, 2), 12),
-                                                        ((1, 2), (2, 4), 6),
-                                                        ((1,), (2,), 1)])
-def test_compute_tuple_from_idx_linear(idx_tuple, dimensions, idx_lin_ref):
-    idx_lin = compute_linear_idx(idx_tuple, dimensions)
-    assert idx_lin == idx_lin_ref
-    idx_tuple2 = compute_tuple_idx(idx_lin, dimensions)
-    assert idx_tuple == idx_tuple2
+    return tuple(idx_tuple)
\ No newline at end of file
