@@ -3,10 +3,10 @@ from typing import Union, Callable
 
 import numpy as np
 
-__author__ = "piveloper"
-__copyright__ = "26.03.2020, piveloper"
+__author__ = "P.Mohr"
+__copyright__ = "26.03.2020, P.Mohr"
 __version__ = "1.0"
-__email__ = "piveloper@gmail.com"
+__email__ = "philipp.mohr@tuhh.de"
 __doc__ = """This script includes helpful functions to extended PyOpenCl functionality."""
 
 from pyopencl_extension.types.auto_gen.cl_types import ClTypesVector
